package com.app.humfuel.activity

import android.app.Activity
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.app.AlertDialog
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.app.humfuel.R
import com.app.humfuel.fragment.home.AddLocationPinFragment
import com.app.humfuel.fragment.home.HomeFragment
import com.app.humfuel.fragment.home.OrderListFragment
import com.app.humfuel.fragment.home.ProfileFragment
import com.app.humfuel.retrofitManager.ResponseListener
import com.app.humfuel.utils.Const
import com.app.humfuel.utils.LocationManager
import com.app.humfuel.utils.setStatusBarTranslucent
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.toolbar_custom.*
import retrofit2.Call

/**
 * Created by neeraj on 15/6/17.
 */
class MainActivity : BaseActivity() {

    private var logoutCall: Call<Any>? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        initUI()
    }

    private fun initUI() {
        navigationView.itemIconTintList = null
        navigationView.setNavigationItemSelectedListener { menuItem ->
            // This method will trigger on item Click of navigation menu
            drawer.closeDrawers()
            var fragment: Fragment? = null
            when (menuItem.itemId) {
                R.id.home -> {
                    fragment = HomeFragment()
                    menuItem.isChecked = true
                }
                R.id.orders -> {
                    fragment = OrderListFragment()
                    menuItem.isChecked = true
                }
                R.id.about -> showAboutDialog()
                R.id.logout -> {
                    logoutCall = apiInterface.logoutUser(store.getInt(Const.USER_ID, 0).toString())
                    apiManager.makeApiCall(logoutCall, this)
                    GoogleSignIn.getClient(this, GoogleSignInOptions.DEFAULT_SIGN_IN).signOut()
                }
                R.id.exit -> finish()
            }

            if (fragment != null)
                supportFragmentManager
                        .beginTransaction()
                        .replace(R.id.frame_container, fragment)
                        .commit()
            true
        }

        val view = navigationView.getHeaderView(0)
        setUserData()
        view.setOnClickListener {
            drawer.closeDrawers()
            supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container, ProfileFragment())
                    .commit()
        }

        if (!intent.getBooleanExtra("isPush", false))
            gotoHome()
        else {
            navigationView.setCheckedItem(R.id.orders)
            val fragment = OrderListFragment()
            val bundle = Bundle()
            bundle.putString("orderId", intent.getStringExtra("orderId"))
            fragment.arguments = bundle
            supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container, fragment)
                    .commit()
        }
        setStatusBarTranslucent(true, this)

        backTBIV.setOnClickListener { onBackPressed() }
        menuTBIV.setOnClickListener {
            hideSoftKeyboard()
            drawer.openDrawer(Gravity.START)
        }
    }

    private fun showAboutDialog() {
        val builder = AlertDialog.Builder(this)
        builder.setTitle("Contact Us")
        val view = View.inflate(this, R.layout.dialog_contact_us, null)
        builder.setView(view)
        val callIV = view.findViewById<ImageView>(R.id.callIV)
        val call1IV = view.findViewById<ImageView>(R.id.call1IV)
        val emailIV = view.findViewById<ImageView>(R.id.emailIV)
        callIV.setOnClickListener { sendCallIntent("+919873524365") }
        call1IV.setOnClickListener { sendCallIntent("+919560052902") }
        emailIV.setOnClickListener {
            val emailIntent = Intent(Intent.ACTION_SENDTO, Uri.fromParts("mailto", "humsafarindia1@gmail.com", null))
            startActivity(Intent.createChooser(emailIntent, "Open Using"))
        }
        builder.create().show()
    }

    private fun sendCallIntent(callNum: String) {
        val callIntent = Intent(Intent.ACTION_DIAL)
        callIntent.data = Uri.parse("tel:$callNum")
        startActivity(Intent.createChooser(callIntent, "Call using"))
    }

    fun setUserData() {
        navigationView.getHeaderView(0).findViewById<TextView>(R.id.nameTV).text = getString(R.string.hello_user, store.getString(Const.USER_NAME, "User"))
    }

    fun setToolbar(title: String, showBack: Boolean) {
        menuTBIV.visibility = if (showBack) View.INVISIBLE else View.VISIBLE
        backTBIV.visibility = if (showBack) View.VISIBLE else View.GONE
        titleTBTV.text = title
    }

    override fun onBackPressed() {
        if (drawer.isDrawerOpen(Gravity.START)) {
            drawer.closeDrawers()
        } else {
            val fragment = supportFragmentManager.findFragmentById(R.id.frame_container)
            if (fragment is HomeFragment) {
                showExit()
            } else if (supportFragmentManager.backStackEntryCount > 0) {
                supportFragmentManager.popBackStack()
            } else {
                gotoHome()
            }
        }
    }

    fun gotoHome() {
        navigationView.setCheckedItem(R.id.home)
        supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container, HomeFragment())
                .commit()
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        super.onSuccess(call, `object`)
        store.setInt(Const.USER_ID, 0)
        startActivity(Intent(this, LoginSignupActivity::class.java))
        finish()
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String, listener: ResponseListener) {
        store.setInt(Const.USER_ID, 0)
        startActivity(Intent(this, LoginSignupActivity::class.java))
        finish()
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        val fragment = supportFragmentManager.findFragmentById(R.id.frame_container)

        if (fragment is AddLocationPinFragment) {
            when (requestCode) {
                LocationManager.REQUEST_LOCATION ->
                    when (resultCode) {
                        Activity.RESULT_OK -> {
                            fragment.onGPSEnable()
                        }
                        Activity.RESULT_CANCELED -> {
                            fragment.onGPSEnableDenied()
                        }
                        else -> {
                            fragment.onGPSEnableDenied()
                        }
                    }
            }
        }
    }
}