package com.app.humfuel.adapter

import android.os.Bundle
import android.support.v7.app.AlertDialog
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import com.app.humfuel.R
import com.app.humfuel.activity.BaseActivity
import com.app.humfuel.data.OrderData
import com.app.humfuel.fragment.home.PendingOrderFragment
import com.app.humfuel.fragment.home.UpdateOrderFragment
import com.app.humfuel.utils.Const
import com.app.humfuel.utils.Utils
import java.util.*


class OrdersAdapter : RecyclerView.Adapter<OrdersAdapter.MyViewHolder> {

    internal var baseActivity: BaseActivity
    private var orderDatas: ArrayList<OrderData>
    private var pendingOrderFragment: PendingOrderFragment? = null


    constructor(baseActivity: BaseActivity, orderDatas: ArrayList<OrderData>, pendingOrderFragment: PendingOrderFragment) {
        baseActivity.log("List size  >>>> " + orderDatas.size)
        this.orderDatas = orderDatas
        this.baseActivity = baseActivity
        this.pendingOrderFragment = pendingOrderFragment
    }

    constructor(baseActivity: BaseActivity, orderDatas: ArrayList<OrderData>) {
        this.orderDatas = orderDatas
        this.baseActivity = baseActivity
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyViewHolder {
        val itemView = LayoutInflater.from(parent.context).inflate(R.layout.adapter_order_list, parent, false)
        return MyViewHolder(itemView)
    }

    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {

        val orderData = orderDatas[holder.adapterPosition]
        holder.orderTV.text = baseActivity.getString(R.string.order_id, orderData.queryNo)
        holder.timeTV.text = Utils.changeDateFormat(orderData.deliveryDate, "yyyy-MM-dd'T'HH:mm:ss", "MMM, dd hh:mm a")
        holder.statusTV.text = orderData.orderStatus
        if (orderData.orderStatus == Const.OrderStatus.CANCELLED) {
            holder.statusTV.append("\nReason : " + orderData.reason)
        }
        holder.addressTV.text = orderData.address
        holder.itemTV.text = orderData.item
        holder.quantityTV.text = baseActivity.getString(R.string.litre, orderData.quantity)
        holder.priceTV.text = baseActivity.getString(R.string.money_unit, calculatePrice(orderData))

        baseActivity.log("OrderId : " + orderData.id + " Status : " + orderData.orderStatus)

        holder.bottomLL.visibility = if (pendingOrderFragment == null) View.GONE else View.VISIBLE
        holder.cancelBT.tag = orderData
        holder.cancelBT.setOnClickListener { view ->
            val orderData1 = view.tag as OrderData
            showCancelDialog(orderData1)
        }
        holder.updateBT.tag = orderData
        holder.updateBT.setOnClickListener { view ->
            val orderData1 = view.tag as OrderData
            val bundle = Bundle()
            bundle.putParcelable("data", orderData1)
            val fragment = UpdateOrderFragment()
            fragment.arguments = bundle
            baseActivity.supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container, fragment)
                    .addToBackStack(null)
                    .commit()
        }
    }


    private fun calculatePrice(orderData: OrderData): String {
        val quantity = Integer.parseInt(orderData.quantity)
        val itemPrice = orderData.itemPrice
        val tax = (orderData.transportationCharges * orderData.tax / 100).toFloat()
        val transportCharge = orderData.transportationCharges.toFloat()

        baseActivity.log(" \nItem : " + quantity * itemPrice + "\nTax : " + tax + "\nTrans : " + transportCharge + "\nTotal : " + Math.round(quantity * itemPrice + tax + transportCharge) + "\n\n\n\n")

        return Math.round(quantity * itemPrice + tax + transportCharge).toString()
    }

    private fun showCancelDialog(orderData1: OrderData) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Cancel Order")
        val view = View.inflate(baseActivity, R.layout.dialog_cancel_order, null)
        builder.setView(view)
        val cancelET: EditText = view.findViewById(R.id.cancelET)
        builder.setPositiveButton("Confirm") { dialogInterface, i ->
            if (cancelET.text.toString().trim { it <= ' ' }.isEmpty()) {
                baseActivity.showToast("Please enter cancel reason", true)
            } else {
                pendingOrderFragment!!.cancelOrder(cancelET.text.toString().trim { it <= ' ' }, orderData1)
                dialogInterface.dismiss()
            }
        }
        builder.setNegativeButton("Cancel") { dialogInterface, i -> dialogInterface.dismiss() }
        builder.show()
    }

    override fun getItemCount() = orderDatas.size

    inner class MyViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        var addressTV: TextView = view.findViewById(R.id.addressTV)
        var timeTV: TextView = view.findViewById(R.id.timeTV)
        var orderTV: TextView = view.findViewById(R.id.orderTV)
        var statusTV: TextView = view.findViewById(R.id.statusTV)
        var itemTV: TextView = view.findViewById(R.id.itemTV)
        var quantityTV: TextView = view.findViewById(R.id.quantityTV)
        var priceTV: TextView = view.findViewById(R.id.priceTV)
        var cancelBT: Button = view.findViewById(R.id.cancelBT)
        var updateBT: Button = view.findViewById(R.id.updateBT)
        var bottomLL: LinearLayout = view.findViewById(R.id.bottomLL)
    }
}