package com.app.humfuel.fragment

import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.app.AlertDialog
import android.view.View
import android.widget.AdapterView
import android.widget.CompoundButton
import android.widget.TextView
import com.app.humfuel.R
import com.app.humfuel.activity.BaseActivity
import com.app.humfuel.activity.MainActivity
import com.app.humfuel.retrofitManager.ApiClient
import com.app.humfuel.retrofitManager.ApiInterface
import com.app.humfuel.retrofitManager.ApiManager
import com.app.humfuel.retrofitManager.ResponseListener
import retrofit2.Call


/**
 * Created by Neeraj Narwal\neeraj.narwal on 2/2/16.
 */
open class BaseFragment : Fragment(), AdapterView.OnItemClickListener, View.OnClickListener, AdapterView.OnItemSelectedListener, CompoundButton.OnCheckedChangeListener, ResponseListener {

    lateinit var baseActivity: BaseActivity
    lateinit var apiInterface: ApiInterface
    lateinit var apiManager: ApiManager

    protected var vieww: View? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        baseActivity = activity as BaseActivity

        baseActivity.hideSoftKeyboard()
        apiManager = ApiManager.getInstance()
        apiManager.updateProgressDialog(baseActivity.progressDialog)
        apiInterface = ApiClient.getClient().create(ApiInterface::class.java)
    }

    fun setToolbar(title: String, showBack: Boolean) {
        (baseActivity as MainActivity).setToolbar(title, showBack)
    }

    override fun onResume() {
        super.onResume()
        activity?.invalidateOptionsMenu()
    }

    protected fun getText(textView: TextView): String {
        return textView.text.toString().trim { it <= ' ' }
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.backIV -> baseActivity.onBackPressed()
        }
    }

    fun showToast(msg: String) {
        baseActivity.showToast(msg, false)
    }

    fun showToast(msg: String, isError: Boolean) {
        baseActivity.showToast(msg, isError)
    }

    fun log(s: String) {
        baseActivity.log(s)
    }

    fun gotoMainAct() {
        startActivity(Intent(baseActivity, MainActivity::class.java))
        baseActivity.finish()
    }

    override fun onItemClick(parent: AdapterView<*>, view: View, position: Int, id: Long) {

    }

    override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {

    }

    override fun onNothingSelected(parent: AdapterView<*>) {

    }

    override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {

    }

    fun isValidMail(email: String): Boolean {
        return email.matches("[a-zA-Z0-9._-]+@[a-z]+\\.+[a-z]+".toRegex())
    }

    fun isValidPassword(password: String): Boolean {
        return password.matches("^(?=\\S+$).{8,}$".toRegex())
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {}

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, responseListener: ResponseListener) {
        if (errorMessage != null && errorMessage == "socketTimeout") {
            showRetry(call, responseListener)
        } else {
            showToast("Error : " + errorMessage!!, true)
            call.cancel()
        }
    }

    private fun showRetry(call: Call<*>, responseListener: ResponseListener) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Alert !!")
        builder.setMessage("Request timeout")
        builder.setPositiveButton("Retry") { dialogInterface, i ->
            val call1 = call.clone()
            apiManager.makeApiCall(call1, responseListener)
        }
        builder.show()
    }

//    fun buildAlertMessageNoGps() {
//        if (activity != null && !activity!!.isFinishing) {
//            val gpsAlert: AlertDialog
//            val alert = AlertDialog.Builder(baseActivity)
//            alert.setMessage(baseActivity.getString(R.string.gps_is_disabled))
//                    .setCancelable(false)
//                    .setPositiveButton(baseActivity.getString(R.string.yes)) { dialog, id ->
//                        dialog.dismiss()
//                        startActivity(Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS))
//                    }
//                    .setNegativeButton(baseActivity.getString(R.string.no)) { dialog, id -> dialog.dismiss() }
//
//            gpsAlert = alert.create()
//            gpsAlert.show()
//        }
//    }

//    fun getLastKnownLocation(locationManager: LocationManager): Location? {
//        val providers = locationManager.getProviders(true)
//        var bestLocation: Location? = null
//        if (ActivityCompat.checkSelfPermission(baseActivity, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(baseActivity, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
//            return null
//        }
//        for (provider in providers) {
//            var l: Location? = locationManager.getLastKnownLocation(provider)
//            if (l == null) {
//                //                locationManager.requestLocationUpdates(provider, 1000L, 10f, listener);
//                l = locationManager.getLastKnownLocation(provider)
//            }
//            if (l != null && (bestLocation == null || l.accuracy > bestLocation.accuracy)) {
//                bestLocation = l
//            }
//        }
//        return bestLocation
//    }
}
