package com.app.humfuel.fragment.home

import android.app.DatePickerDialog
import android.app.TimePickerDialog
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.os.Handler
import android.support.v7.app.AlertDialog
import android.text.Html
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import com.app.humfuel.R
import com.app.humfuel.activity.MainActivity
import com.app.humfuel.data.OrderCreatingData
import com.app.humfuel.data.OrderSubmitData
import com.app.humfuel.data.PreorderData
import com.app.humfuel.data.TermsData
import com.app.humfuel.fragment.BaseFragment
import com.app.humfuel.fragment.OTPHandle
import com.app.humfuel.retrofitManager.ResponseListener
import com.app.humfuel.utils.Const
import com.app.humfuel.utils.PinAdd
import com.app.humfuel.utils.Utils
import com.app.humfuel.utils.myTextWatcher
import com.google.android.gms.maps.model.LatLng
import kotlinx.android.synthetic.main.fg_home.*
import retrofit2.Call
import java.math.BigDecimal
import java.math.RoundingMode
import java.text.SimpleDateFormat
import java.util.*

/**
 * Created by neeraj on 14/6/17.
 */
class HomeFragment : BaseFragment() {
    private var currentCal = Calendar.getInstance()
    private val cal = Calendar.getInstance()
    private var preorderDataCall: Call<PreorderData>? = null
    private var reportCall: Call<OrderSubmitData>? = null
    private var otpVerifyCall: Call<OrderSubmitData>? = null
    private var preorderData: PreorderData? = null
    private var latLng: LatLng? = null
    private var address: String? = ""
    private var pincodeToPost: String? = null
    private var termsCall: Call<TermsData>? = null
    private var termsData: TermsData? = null
    private var alertDialog: AlertDialog? = null

    private val fuelPrice: Float
        get() {
            return if (fueltypeSP.selectedItemPosition == 0) {
                showToast("Please select fuel type", true)
                0f
            } else if (fueltypeSP.selectedItemPosition == 1) {
                if (preorderData!!.dieselAvailable) preorderData!!.dieselPrice else preorderData!!.petrolPrice
            } else
                preorderData!!.petrolPrice
        }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return vieww ?: inflater.inflate(R.layout.fg_home, null)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        vieww = view
        initUI()
        setData()
        if (termsData == null) {
            termsCall = apiInterface.terms
            apiManager.makeApiCall(termsCall, this)
        }
    }

    override fun onResume() {
        super.onResume()
        setToolbar(getString(R.string.app_name), false)
    }

    private var watcher: TextWatcher? = null

    override fun onPause() {
        super.onPause()
        litresET.removeTextChangedListener(watcher)
    }

    private fun initUI() {
        watcher = litresET.myTextWatcher { calculateCharges() }
        submitBT.setOnClickListener { if (validate()) showTerms() }
        dateTimeTV.setOnClickListener { selectDate() }
        checkPincodeBT.setOnClickListener {
            if (getText(pincodeET).isEmpty()) showToast("Please enter pincode", true)
            else getPreorderData()
        }
        mapIV.setOnClickListener {
            addLocationPin(AddLocationPinFragment())
        }
        Handler().postDelayed({ addressET.setText(address) }, 100)
    }

    private fun showTerms() {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Terms and Conditions")
        builder.setMessage(Html.fromHtml(termsData!!.terms))
        builder.setPositiveButton("Accept") { dialogInterface, i -> submitOrder() }
        builder.show()
    }

    private fun getPreorderData() {
        if (pincodeET.text.toString().trim().isEmpty()) {
            showToast("Please enter pincode and check availability", true)
        } else {
            preorderDataCall = apiInterface.preorderData(pincodeET.text.toString().trim())
            apiManager.makeApiCall(preorderDataCall, this)
        }
    }

    private fun calculateCharges() {
        if (preorderData != null) {
            val litres = if (litresET.text.toString().trim().isEmpty()) 0 else Integer.parseInt(litresET.text.toString())
            if (litres == 0) {
                resetFields()
            } else {
                val fuelPrice = fuelPrice
                if (fuelPrice != 0f) {
                    val subTotal = BigDecimal((litres * fuelPrice).toDouble()).setScale(2, RoundingMode.FLOOR).toFloat()
                    val tax = BigDecimal(preorderData!!.transportationCharges * preorderData!!.serviceTax / 100).setScale(2, RoundingMode.FLOOR).toFloat()
                    val total = BigDecimal((preorderData!!.transportationCharges + tax).toDouble()).setScale(2, RoundingMode.FLOOR).toFloat()
                    subtotalTV.text = "₹" + subTotal.toString()
                    transportTV.text = "₹" + preorderData!!.transportationCharges.toString()
                    taxTV.text = "₹" + tax.toString()
                    totalTV.text = "₹" + total.toString()
                    grandTotalTV.text = "₹" + BigDecimal((subTotal + total).toDouble()).setScale(2, RoundingMode.FLOOR).toString()
                } else {
                    resetFields()
                }
            }
        }
    }

    private fun resetFields() {
        subtotalTV.text = ""
        transportTV.text = ""
        taxTV.text = ""
        totalTV.text = ""
        grandTotalTV.text = ""
    }

    private fun selectDate() {

        val datePickerDialog = DatePickerDialog(baseActivity,
                DatePickerDialog.OnDateSetListener { view, year, monthOfYear, dayOfMonth ->
                    cal.set(year, monthOfYear, dayOfMonth)
                    if (currentCal.after(cal)) {
                        showToast("This is a past date", true)
                    } else {
                        dateTimeTV.text = String.format(Locale.US, "%d-%d-%d", dayOfMonth, monthOfYear + 1, year)
                        selectTime()
                    }
                }, cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DAY_OF_MONTH))
        datePickerDialog.show()
    }

    private fun selectTime() {
        val timePickerDialog = TimePickerDialog(baseActivity,
                TimePickerDialog.OnTimeSetListener { view, hourOfDay, minute ->
                    cal.set(Calendar.HOUR_OF_DAY, hourOfDay)
                    cal.set(Calendar.MINUTE, minute)
                    if (currentCal.after(cal)) {
                        showToast("This is Past time", true)
                        dateTimeTV.text = ""
                    } else
                        dateTimeTV.append(" " + Utils.changeDateFormat(hourOfDay.toString() + ":" + minute, "HH:mm", "hh:mm a"))
                }, cal.get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE), false)
        timePickerDialog.show()
    }

    private fun validate(): Boolean {
        when {
            addressET.text.toString().trim().isEmpty() -> showToast("Please enter delivery address", true)
            latLng == null -> showDialogPinAdd()
            pincodeET.text.toString().trim().isEmpty() -> showToast("Please enter pincode")
            fueltypeSP.selectedItemPosition == 0 -> showToast("Please select fuel type", true)
            dateTimeTV.text.toString().trim().isEmpty() -> showToast("Please enter delivery time and date", true)
            litresET.text.toString().trim().isEmpty() -> showToast("Please enter quantity of fuel required", true)
            else -> return true
        }
        return false
    }

    private fun showDialogPinAdd() {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Add Location Pin")
        builder.setMessage("Please set a Location pin on " + addressET.text.toString().trim())
        builder.setPositiveButton("Add") { dialogInterface, _ ->
            val bundle = Bundle()
            bundle.putBoolean("isPin", true)
            bundle.putString("address", addressET.text.toString().trim())
            val fragment = AddLocationPinFragment()
            fragment.arguments = bundle
            addLocationPin(fragment)
        }
        builder.create().show()
    }

    private fun addLocationPin(fragment: AddLocationPinFragment) {
        PinAdd.setListener { address, latLng ->
            this.address = address
            this.latLng = latLng
//            addressET.setText(address)
        }
        address = if (getText(addressET).isNotEmpty()) addressET.text.toString() else ""
        baseActivity.supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container, fragment)
                .addToBackStack(null)
                .commitAllowingStateLoss()
    }

    private fun submitOrder() {
        val orderCreatingData = OrderCreatingData()
        orderCreatingData.userId = baseActivity.store.getInt(Const.USER_ID).toString()
        orderCreatingData.address = getText(addressET)
        orderCreatingData.latitude = latLng!!.latitude.toString()
        orderCreatingData.longitude = latLng!!.longitude.toString()
        orderCreatingData.pincode = pincodeToPost
        orderCreatingData.item = (fueltypeSP.selectedItem as String).replace("₹", "")
        orderCreatingData.deliveryDate = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US).format(cal.time)
        orderCreatingData.quantity = getText(litresET)
        orderCreatingData.itemPrice = (if (fueltypeSP.selectedItemPosition == 1) preorderData!!.dieselPrice else preorderData!!.petrolPrice).toString()
        orderCreatingData.transportationCharges = preorderData!!.transportationCharges.toString()
        orderCreatingData.tax = preorderData!!.serviceTax.toString()
        orderCreatingData.orderStatus = Const.OrderStatus.PENDING

        reportCall = apiInterface.submitOrder(orderCreatingData)
        apiManager.makeApiCall(reportCall, this)
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        if (reportCall === call) {
            val orderSubmitData = `object` as OrderSubmitData
            showAlert(orderSubmitData)
            showToast("Request have been submitted")
        } else if (preorderDataCall === call) {
            showToast("Available to this pincode")
            preorderData = `object` as PreorderData
            setData()
            calculateCharges()
        } else if (otpVerifyCall === call) {
            if (activity != null && !activity!!.isFinishing && alertDialog != null && alertDialog!!.isShowing)
                alertDialog!!.dismiss()
            showToast("Your order has been placed")
        } else if (termsCall === call) {
            termsData = `object` as TermsData
        }
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, listener: ResponseListener) {
        super.onError(call, statusCode, errorMessage, listener)
        if (preorderDataCall === call) {
            preorderData = null
            setData()
            resetFields()
        }
    }

    private fun setData() {
        val list = ArrayList<String>()
        list.add("Select Fuel Type")
        if (preorderData != null) {
            pincodeToPost = getText(pincodeET)
            if (preorderData!!.dieselAvailable) {
                list.add("Diesel @ ₹" + preorderData!!.dieselPrice + "/L")
            }
            if (preorderData!!.petrolAvailable) {
                list.add("Petrol @ ₹" + preorderData!!.petrolPrice + "/L")
            }
        }
        val adp1 = ArrayAdapter(baseActivity, android.R.layout.simple_list_item_1, list)
        adp1.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        fueltypeSP.adapter = adp1
        fueltypeSP.onItemSelectedListener = if (preorderData != null) this else null
        fueltypeSP.isEnabled = preorderData != null
        taxPercTV.text = if (preorderData != null) baseActivity.getString(R.string.tax_perc, preorderData!!.serviceTax) else "GST"
    }

    override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
        calculateCharges()
    }

    private fun showAlert(orderSubmitData: OrderSubmitData) {
        val builder = AlertDialog.Builder(baseActivity)
        val view = View.inflate(baseActivity, R.layout.dialog_otp_order, null)
        builder.setView(view)
        val verifyET1: EditText = view.findViewById(R.id.verifyET1)
        val verifyET2: EditText = view.findViewById(R.id.verifyET2)
        val verifyET3: EditText = view.findViewById(R.id.verifyET3)
        val verifyET4: EditText = view.findViewById(R.id.verifyET4)
        val submitBT: Button = view.findViewById(R.id.submitBT)

        OTPHandle.readOTP(baseActivity, { otp -> validateOTP(otp, orderSubmitData) }, submitBT, verifyET1, verifyET2, verifyET3, verifyET4)

        builder.setOnDismissListener { (baseActivity as MainActivity).gotoHome() }
        builder.setCancelable(false)
        alertDialog = builder.create()
        alertDialog!!.show()
        alertDialog!!.window!!.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))

    }

    private fun validateOTP(otp: String, orderSubmitData: OrderSubmitData) {
        otpVerifyCall = apiInterface.authenticateOrder(orderSubmitData.id.toString(), otp)
        apiManager.makeApiCall(otpVerifyCall, this)
    }
}