package com.app.humfuel.fragment.home

import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.app.humfuel.R
import com.app.humfuel.data.OrderData
import com.app.humfuel.fragment.BaseFragment
import com.app.humfuel.retrofitManager.ResponseListener
import com.app.humfuel.utils.Const
import com.app.humfuel.utils.Const.OrderStatus.*
import kotlinx.android.synthetic.main.inc_selector_button.*
import retrofit2.Call
import java.util.*

/**
 * Created by neeraj on 24/7/17.
 */
class OrderListFragment : BaseFragment() {

    private val completedOrders = ArrayList<OrderData>()
    private val pendingOrders = ArrayList<OrderData>()

    private var orderIdToShowOnTop: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (arguments != null && arguments!!.containsKey("orderId")) {
            orderIdToShowOnTop = arguments!!.getString("orderId")
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        setToolbar("Orders", true)
        return inflater.inflate(R.layout.fg_order_list, null)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
        getOrderList()
    }

    private fun initUI() {
        text1TV.setOnClickListener { setLayout(1) }
        text2TV.setOnClickListener { setLayout(2) }
    }

    fun getOrderList() {
        apiManager.makeApiCall(apiInterface.ordersList(baseActivity.store.getInt(Const.USER_ID).toString()), this)
    }

    private fun setLayout(i: Int) {
        text1TV.setBackgroundColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.White else R.color.transparent))
        text2TV.setBackgroundColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.transparent else R.color.White))
        text1TV.setTextColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.Black else R.color.White))
        text2TV.setTextColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.White else R.color.Black))
        val bundle = Bundle()
        bundle.putParcelableArrayList("dataList", if (i == 1) pendingOrders else completedOrders)
        val fragment = if (i == 1) PendingOrderFragment() else CompletedOrderFragment()
        fragment.arguments = bundle
        baseActivity.supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container_tabs, fragment)
                .commit()
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        super.onSuccess(call, `object`)
        pendingOrders.clear()
        completedOrders.clear()
        val orderDatas = `object` as ArrayList<OrderData>
        for (orderData in orderDatas) {
            if (orderData.orderStatus == PENDING || orderData.orderStatus == DISPATCHED) {
                pendingOrders.add(orderData)
            } else {
                completedOrders.add(orderData)
            }
        }
        if (orderIdToShowOnTop != null) {
            var foundInCompletes = false
            for (orderData in completedOrders) {
                if (orderIdToShowOnTop == orderData.id) {
                    completedOrders.remove(orderData)
                    completedOrders.add(0, orderData)
                    foundInCompletes = true
                    break
                }
            }
            if (!foundInCompletes) {
                for (orderData in pendingOrders) {
                    if (orderIdToShowOnTop == orderData.id) {
                        pendingOrders.remove(orderData)
                        pendingOrders.add(0, orderData)
                        break
                    }
                }
                text1TV.callOnClick()
            } else
                text2TV.callOnClick()
        } else
            text1TV.callOnClick()
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, responseListener: ResponseListener) {
        super.onError(call, statusCode, errorMessage, responseListener)
        //TODO Message key is not in error json
        text1TV.callOnClick()
    }
}
