package com.app.humfuel.fragment.login

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.app.humfuel.BuildConfig
import com.app.humfuel.R
import com.app.humfuel.data.SignupData
import com.app.humfuel.data.UserData
import com.app.humfuel.fragment.BaseFragment
import com.app.humfuel.retrofitManager.ResponseListener
import com.app.humfuel.utils.Const
import com.app.humfuel.utils.FacebookLogin
import com.app.humfuel.utils.GPlusLoginActivity
import com.app.humfuel.utils.SocialLogin
import kotlinx.android.synthetic.main.fg_login.*
import retrofit2.Call

/**
 * Created by neeraj on 14/6/17.
 */
class LoginFragment : BaseFragment() {
    private var loginCall: Call<UserData>? = null
    private var socialLoginCall: Call<UserData>? = null
    private var loginType: String? = null
    private var socialLoginSignupData: SignupData? = null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fg_login, null)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
    }

    private fun initUI() {
        loginBT.setOnClickListener { if (validate()) gotoLogin() }
        singupBT.setOnClickListener { gotoSignUp() }
        forgotIV.setOnClickListener {
            baseActivity.supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container_ls, ForgotPasswordFragment())
                    .addToBackStack(null)
                    .commit()
        }
        fbIV.setOnClickListener {
            SocialLogin.doFbSignin { graph ->
                val jsonObject = graph.jsonObject
                log(jsonObject.toString() + "")
                val signupData = SignupData()
                signupData.email = jsonObject.optString("email")
                signupData.password = jsonObject.optString("id")
                signupData.name = jsonObject.optString("first_name") + " " + jsonObject.optString("last_name")
                if (signupData.email == null || signupData.email.isEmpty()) {
                    showToast("Unable to fetch data from Facebook, maybe you didn't give permission to get your email address")
                } else {
                    loginToServer(signupData, Const.LoginType.FACEBOOK)
                }
            }
            startActivity(Intent(baseActivity, FacebookLogin::class.java))
        }
        googleIV.setOnClickListener {
            SocialLogin.doGSignin { googleSignInAccount ->
                val signupData = SignupData()
                signupData.email = googleSignInAccount.email
                signupData.password = googleSignInAccount.id
                signupData.name = googleSignInAccount.displayName
                if (signupData.email.isEmpty()) {
                    showToast("Unable to fetch data from Google, maybe you didn't give permission to get your email address")
                } else {
                    loginToServer(signupData, Const.LoginType.GOOGLE)
                }
            }
            startActivity(Intent(baseActivity, GPlusLoginActivity::class.java))
        }
        if (BuildConfig.DEBUG) {
            emailET.setText("test@test.com")
            passwordET.setText("admin123")
        }
    }

    private fun gotoLogin() {
        loginType = Const.LoginType.DEFAULT
        loginCall = apiInterface.login(getText(emailET), getText(passwordET), loginType, baseActivity.store.getString(Const.DEVICE_TOKEN))
        apiManager.makeApiCall(loginCall, this)
    }

    private fun validate(): Boolean {
        when {
            getText(emailET).isEmpty() -> showToast("Please enter email", true)
            getText(passwordET).isEmpty() -> showToast("Please enter password", true)
            else -> return true
        }
        return false
    }

    private fun gotoSignUp() {
        baseActivity.supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container_ls, SignUpFragment())
                .addToBackStack(null)
                .commit()
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        if (loginCall === call || socialLoginCall === call) {
            val userData = `object` as UserData
            baseActivity.store.saveUserData(Const.USER_DATA, userData)
            baseActivity.store.setInt(Const.USER_ID, userData.userId)
            baseActivity.store.saveString(Const.USER_NAME, userData.name)
            gotoMainAct()
        }
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, listener: ResponseListener) {
        if (loginType == Const.LoginType.DEFAULT) {
            super.onError(call, statusCode, errorMessage, listener)
            if (statusCode == 405) {
                val bundle = Bundle()
                bundle.putBoolean("isFromLogin", true)
                bundle.putString("email", getText(emailET))
                val fragment = OTPFragment()
                fragment.arguments = bundle
                baseActivity.supportFragmentManager
                        .beginTransaction()
                        .replace(R.id.frame_container_ls, fragment)
                        .addToBackStack(null)
                        .commit()
                baseActivity.hideSoftKeyboard()
            }
        } else if (statusCode == 409) {
            socialLoginSignupData!!.loginType = loginType
            val bundle = Bundle()
            bundle.putParcelable("data", socialLoginSignupData)
            val fragment = SocialLoginSignupFragment()
            fragment.arguments = bundle
            baseActivity.supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container_ls, fragment)
                    .addToBackStack(null)
                    .commit()
        } else {
            showToast("Network Problem", true)
        }
    }

    private fun loginToServer(signupData: SignupData, loginType: String) {
        socialLoginSignupData = signupData
        this.loginType = loginType
        socialLoginCall = apiInterface.login(signupData.email, signupData.password, loginType, baseActivity.store.getString(Const.DEVICE_TOKEN))
        apiManager.makeApiCall(socialLoginCall, this)
    }
}