package com.app.humfuel.retrofitManager;

import com.app.humfuel.utils.Utils;
import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.HashMap;

import okhttp3.RequestBody;
import okio.Buffer;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static com.app.humfuel.utils.Const.ErrorCodes.NO_INTERNET;
import static com.app.humfuel.utils.Const.ErrorCodes.SERVER_ERROR;
import static com.app.humfuel.utils.Const.ErrorCodes.SOCKET_TIMEOUT;

/**
 * Created by Neeraj Narwal on 31/5/17.
 */
public class ApiManager implements Callback {

    static final String TAG = ApiManager.class.getSimpleName();
    private static ApiManager apiManager;
    private ProgressDialog progressDialog;
    private HashMap<Call, ResponseListener> apiResponseHashMap = new HashMap<>();

    public static ApiManager getInstance() {
        if (apiManager == null) {
            apiManager = new ApiManager();
        }
        return apiManager;
    }

    public void updateProgressDialog(ProgressDialog progressDialog) {
        this.progressDialog = progressDialog;
    }

    //Constructor when Progress message need to update
    public void makeApiCall(Call call, String progressMessage, ResponseListener responseListener) {
        if (progressMessage != null && !progressMessage.isEmpty()) {
            progressDialog.updateMessage(progressMessage);
        }
        makeApiCall(call, true, responseListener);
        HashMap<String, String> mmap = new HashMap<>();
    }

    public void makeApiCall(Call call, ResponseListener responseListener) {
        makeApiCall(call, true, responseListener);
    }

    public void makeApiCall(Call call, boolean showProgress, ResponseListener responseListener) {
        try {
            apiResponseHashMap.put(call, responseListener);
            call.enqueue(this);
            if (showProgress) {
                progressDialog.startProgressDialog();
            }

            //Logs post URL
            log(call.request().url() + "");

            //Logs post params of Multipart request
            log("Post Params >>>> \n" + bodyToString(call.request().body())
                    .replace("\r", "")
                    .replaceAll("--+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                    .replaceAll("Content+[a-zA-Z0-9-\\/:=;\\ ]+;\\s", "")
                    .replaceAll("Content+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                    .replaceAll("charset+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                    .replace("name=", "")
                    .replace("\n\n", "--> ")
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void log(String string) {
        Utils.debugLog(TAG, string);
    }

    private String bodyToString(final RequestBody request) {
        try {
            Buffer buffer = new Buffer();
            if (request != null)
                request.writeTo(buffer);
            else
                return "";
            return buffer.readUtf8();
        } catch (final IOException e) {
            return "did not work";
        }
    }

    @Override
    public void onResponse(Call call, Response response) {
        progressDialog.stopProgressDialog();
        ResponseListener responseListener = apiResponseHashMap.get(call);
        try {
            if (response.isSuccessful()) {
                Object o = response.body();
                log("Response >>>>>> " + new Gson().toJson(o));
                responseListener.onSuccess(call, o);
            } else {
                JsonParser parser = new JsonParser();
                JsonElement mJson = parser.parse(response.errorBody().string());
                Gson gson = new Gson();
                ErrorData errorResponse = gson.fromJson(mJson, ErrorData.class);
                log(errorResponse.message + "");
                responseListener.onError(call, response.code(), errorResponse.message, responseListener);
            }
        } catch (Exception ignored) {
        }

        //This handling is to handle a "success" object in response which tell its an error or not like OK/NOK
        //Do update this method if your handling is of different type if not then send response as it is


//        boolean wasError = false;
//        try {
//            JsonElement object = (JsonElement) response.body();
//            log(object + "");
//            if (object != null
//                    && object.getAsJsonObject().has("success")
//                    && object.getAsJsonObject().get("success").getAsString().equals("0")) {
//                responseListener.onError(call, response.code(), object.getAsJsonObject().get("message").getAsString(), responseListener);
//                wasError = true;
//            }
//        } catch (Exception ignored) {
//        }
//        if (!wasError) {
//            responseListener.onSuccess(call, response.body());
//        responseListener.onFinish();
//        }
        apiResponseHashMap.remove(call);
    }

    @Override
    public void onFailure(Call call, Throwable t) {
        progressDialog.stopProgressDialog();
        ResponseListener responseListener = apiResponseHashMap.get(call);
        try {
            if (t.getClass() == SocketTimeoutException.class) {
                responseListener.onError(call, SOCKET_TIMEOUT, "socketTimeout", responseListener);
            } else if (t.getClass() == UnknownHostException.class) {
                responseListener.onError(call, NO_INTERNET, "No Network", responseListener);
            } else {
                responseListener.onError(call, SERVER_ERROR, t.getMessage(), responseListener);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
//        responseListener.onFinish();
        apiResponseHashMap.remove(call);
    }
}
