package com.vendor.humfuel.activity

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.support.v4.app.Fragment
import android.support.v7.app.ActionBarDrawerToggle
import android.support.v7.app.AlertDialog
import android.support.v7.widget.Toolbar
import android.view.Gravity
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.vendor.humfuel.R
import com.vendor.humfuel.fragment.HomeFragment
import com.vendor.humfuel.fragment.ProfileFragment
import com.vendor.humfuel.utils.Const
import com.vendor.humfuel.utils.setStatusBarTranslucent
import kotlinx.android.synthetic.main.activity_main.*

/**
 * Created by neeraj on 15/6/17.
 */
class MainActivity : BaseActivity() {
    private var toolbar: Toolbar? = null
    private var actionBarDrawerToggle: ActionBarDrawerToggle? = null
    private var mToolBarNavigationListenerIsRegistered: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        toolbar = findViewById<View>(R.id.toolbar) as Toolbar
        setSupportActionBar(toolbar)
        navigationView.itemIconTintList = null
        navigationView.setNavigationItemSelectedListener { menuItem ->
            // This method will trigger on item Click of navigation menu
            menuItem.isChecked = true
            drawerLayout.closeDrawers()
            var fragment: Fragment? = null
            when (menuItem.itemId) {
                R.id.home -> fragment = HomeFragment()
                R.id.about -> showAboutDialog()
                R.id.logout -> try {
                    progressDialog.startProgressDialog()
                    Handler().postDelayed({
                        progressDialog.stopProgressDialog()
                        store.setInt(Const.USER_ID, 0)
                        startActivity(Intent(this@MainActivity, LoginSignupActivity::class.java))
                        finish()
                    }, 2000)
                } catch (ignored: Exception) {
                }

                R.id.exit -> finish()
            }

            if (fragment != null)
                supportFragmentManager
                        .beginTransaction()
                        .replace(R.id.frame_container, fragment)
                        .commit()
            true
        }

        val view = navigationView!!.getHeaderView(0)
        setUserData()
        view.setOnClickListener {
            drawerLayout!!.closeDrawers()
            supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container, ProfileFragment())
                    .commit()
        }

        actionBarDrawerToggle = object : ActionBarDrawerToggle(this, drawerLayout, toolbar, R.string.openDrawer, R.string.closeDrawer) {

            override fun onDrawerClosed(drawerView: View) {
                hideSoftKeyboard()
                super.onDrawerClosed(drawerView)
            }

            override fun onDrawerOpened(drawerView: View) {
                hideSoftKeyboard()
                super.onDrawerOpened(drawerView)
            }
        }
        drawerLayout!!.addDrawerListener(actionBarDrawerToggle!!)
        actionBarDrawerToggle!!.syncState()
        if (!intent.getBooleanExtra("isPush", false))
            gotoHome()
        else {
            navigationView!!.setCheckedItem(R.id.home)
            val fragment = HomeFragment()
            val bundle = Bundle()
            bundle.putString("orderId", intent.getStringExtra("orderId"))
            fragment.arguments = bundle
            supportFragmentManager
                    .beginTransaction()
                    .replace(R.id.frame_container, fragment)
                    .commit()
        }
        setStatusBarTranslucent(true, this)
    }

    fun setUserData() {
        navigationView.getHeaderView(0).findViewById<TextView>(R.id.nameTV).text = getString(R.string.hello_user, store.getString(Const.USER_NAME, "User"))
    }

    fun setToolbar(title: String, show: Boolean) {
        toolbar!!.visibility = if (show) View.VISIBLE else View.GONE
        toolbar!!.title = title
    }

    private fun showAboutDialog() {
        val builder = AlertDialog.Builder(this)
        builder.setTitle("Contact Us")
        val view = View.inflate(this, R.layout.dialog_contact_us, null)
        builder.setView(view)
        val callIV = view.findViewById<ImageView>(R.id.callIV)
        val call1IV = view.findViewById<ImageView>(R.id.call1IV)
        val emailIV = view.findViewById<ImageView>(R.id.emailIV)
        callIV.setOnClickListener { sendCallIntent("+919873524365") }
        call1IV.setOnClickListener { sendCallIntent("+919560052902") }
        emailIV.setOnClickListener {
            val emailIntent = Intent(Intent.ACTION_SENDTO, Uri.fromParts("mailto", "humsafarindia1@gmail.com", null))
            startActivity(Intent.createChooser(emailIntent, "Open Using"))
        }
        builder.create().show()
    }


    private fun sendCallIntent(callNum: String) {
        val callIntent = Intent(Intent.ACTION_DIAL)
        callIntent.data = Uri.parse("tel:$callNum")
        startActivity(Intent.createChooser(callIntent, "Call using"))
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        supportActionBar!!.setDisplayHomeAsUpEnabled(false)
        actionBarDrawerToggle!!.isDrawerIndicatorEnabled = true
        actionBarDrawerToggle!!.toolbarNavigationClickListener = null
        mToolBarNavigationListenerIsRegistered = false
        return super.onCreateOptionsMenu(menu)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (actionBarDrawerToggle!!.onOptionsItemSelected(item)) {
            return true
        }
        when (item.itemId) {
            android.R.id.home -> onBackPressed()
        }
        return super.onOptionsItemSelected(item)
    }

    override fun onBackPressed() {
        if (drawerLayout.isDrawerOpen(Gravity.START)) {
            drawerLayout.closeDrawers()
        } else {
            val fragment = supportFragmentManager.findFragmentById(R.id.frame_container)
            if (fragment is HomeFragment) {
                showExit()
            } else if (supportFragmentManager.backStackEntryCount > 0) {
                supportFragmentManager.popBackStack()
            } else {
                gotoHome()
            }
        }
    }

    fun gotoHome() {
        navigationView.setCheckedItem(R.id.home)
        supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container, HomeFragment())
                .commit()
    }
}