package com.vendor.humfuel.adapter

import android.support.v7.app.AlertDialog
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.EditText
import android.widget.LinearLayout
import android.widget.TextView
import com.github.gcacace.signaturepad.views.SignaturePad
import com.vendor.humfuel.R
import com.vendor.humfuel.activity.BaseActivity
import com.vendor.humfuel.data.OrderData
import com.vendor.humfuel.fragment.CompletedOrderFragment
import com.vendor.humfuel.fragment.PendingOrderFragment
import com.vendor.humfuel.utils.Const
import java.math.BigDecimal
import java.math.RoundingMode
import java.util.*


class OrdersAdapter : RecyclerView.Adapter<OrdersAdapter.MyViewHolder> {

    private var baseActivity: BaseActivity
    private var orderDatas: ArrayList<OrderData>
    private var completedOrderFragment: CompletedOrderFragment? = null
    private var pendingOrderFragment: PendingOrderFragment? = null

    constructor(baseActivity: BaseActivity, orderDatas: ArrayList<OrderData>, completedOrderFragment: CompletedOrderFragment) {
        this.orderDatas = orderDatas
        this.baseActivity = baseActivity
        this.completedOrderFragment = completedOrderFragment
    }

    constructor(baseActivity: BaseActivity, orderDatas: ArrayList<OrderData>, pendingOrderFragment: PendingOrderFragment) {
        this.orderDatas = orderDatas
        this.baseActivity = baseActivity
        this.pendingOrderFragment = pendingOrderFragment
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyViewHolder {
        val itemView = LayoutInflater.from(parent.context)
                .inflate(R.layout.adapter_order_list, parent, false)
        return MyViewHolder(itemView)
    }

    override fun onBindViewHolder(holder: MyViewHolder, position: Int) {
        val orderData = orderDatas[holder.adapterPosition]
        holder.orderTV.text = baseActivity.getString(R.string.order_id, orderData.queryNo)
        holder.timeTV.text = baseActivity.changeDateFormat(orderData.deliveryDate, "yyyy-MM-dd'T'HH:mm:ss", "MMM, dd hh:mm a")
        holder.statusTV.text = orderData.orderStatus

        if (orderData.orderStatus == Const.OrderStatus.CANCELLED) {
            holder.statusTV.append("\nReason : " + orderData.reason)
        }

        holder.addressTV.text = orderData.address
        holder.itemTV.text = orderData.item
        holder.quantityTV.text = baseActivity.getString(R.string.litre, orderData.quantity)
        holder.priceTV.text = baseActivity.getString(R.string.money_unit, calculatePrice(orderData))

        baseActivity.log("OrderId : " + orderData.id + " Status : " + orderData.orderStatus)

        holder.bottomLL.visibility = if (completedOrderFragment != null) View.GONE else View.VISIBLE

        holder.cancelBT.tag = orderData
        holder.cancelBT.setOnClickListener { view ->
            val orderData1 = view.tag as OrderData
            showCancelDialog(orderData1)
        }
        if (orderData.orderStatus == Const.OrderStatus.ASSIGNED) {
            holder.updateBT.text = "Dispatch"
        } else {
            holder.updateBT.text = "Complete"
        }
        holder.updateBT.tag = orderData
        holder.updateBT.setOnClickListener { view ->
            val orderData1 = view.tag as OrderData
            if (orderData1.orderStatus == Const.OrderStatus.ASSIGNED) {
                showDispatchDialog(orderData1)
            } else {
                showSignatureDialog(orderData1)
            }
        }
    }

    private fun calculatePrice(orderData: OrderData): String {
        val quantity = Integer.parseInt(orderData.quantity)
        val itemPrice = orderData.itemPrice
        val tax = orderData.transportationCharges * orderData.tax / 100
        val transportCharge = orderData.transportationCharges

        baseActivity.log(" \nItem : " + quantity * itemPrice + "\nTax : " + tax + "\nTrans : " + transportCharge + "\nTotal : " + Math.round(quantity * itemPrice + tax + transportCharge) + "\n\n\n\n")

        return BigDecimal((quantity * itemPrice + tax + transportCharge).toDouble()).setScale(2, RoundingMode.FLOOR).toString()
    }

    private fun showDispatchDialog(orderData1: OrderData) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Dispatch Order")
        builder.setMessage("Are you sure you want to mark this order as dispatched ?")
        builder.setPositiveButton("Yes") { dialogInterface, _ ->
            pendingOrderFragment?.dispatchOrder(orderData1)
            dialogInterface.dismiss()
        }
        builder.setNegativeButton("No", null)
        builder.show()
    }

    private fun showSignatureDialog(orderData1: OrderData) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Complete Order")
        val view = View.inflate(baseActivity, R.layout.dialog_complete_order, null)
        val signSP: SignaturePad = view.findViewById(R.id.signSP)
        signSP.setMaxWidth(4f)
        signSP.setMinWidth(3f)
        builder.setView(view)
        builder.setPositiveButton("Done") { dialogInterface, _ ->
            if (signSP.isEmpty) {
                baseActivity.showToast("Please provide signatures", true)
            } else {
                val bitmap = signSP.signatureBitmap
                pendingOrderFragment?.completeOrder(bitmap, orderData1)
                dialogInterface.dismiss()
            }
        }
        builder.setNegativeButton("Cancel", null)
        builder.show()
    }

    private fun showCancelDialog(orderData1: OrderData) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Cancel Order")
        val view = View.inflate(baseActivity, R.layout.dialog_cancel_order, null)
        builder.setView(view)
        val cancelET: EditText = view.findViewById(R.id.cancelET)
        builder.setPositiveButton("Confirm") { dialogInterface, i ->
            if (cancelET.text.toString().trim { it <= ' ' }.isEmpty()) {
                baseActivity.showToast("Please enter cancel reason", true)
            } else {
                pendingOrderFragment?.cancelOrder(cancelET.text.toString().trim { it <= ' ' }, orderData1)
                dialogInterface.dismiss()
            }
        }
        builder.setNegativeButton("Cancel", null)
        builder.show()
    }

    override fun getItemCount() = orderDatas.size

    inner class MyViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        var addressTV: TextView = view.findViewById(R.id.addressTV)
        var timeTV: TextView = view.findViewById(R.id.timeTV)
        var orderTV: TextView = view.findViewById(R.id.orderTV)
        var statusTV: TextView = view.findViewById(R.id.statusTV)
        var itemTV: TextView = view.findViewById(R.id.itemTV)
        var quantityTV: TextView = view.findViewById(R.id.quantityTV)
        var priceTV: TextView = view.findViewById(R.id.priceTV)
        var cancelBT: Button = view.findViewById(R.id.cancelBT)
        var updateBT: Button = view.findViewById(R.id.updateBT)
        var bottomLL: LinearLayout = view.findViewById(R.id.bottomLL)
    }
}