package com.vendor.humfuel.apiHandle;

import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.vendor.humfuel.BuildConfig;
import com.vendor.humfuel.data.ErrorData;

import java.io.IOException;
import java.net.SocketTimeoutException;

import okhttp3.RequestBody;
import okio.Buffer;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

/**
 * Created by neeraj on 31/5/17.
 */
public class ApiHitAndHandle implements Callback {

    private static final String TAG = ApiHitAndHandle.class.getSimpleName();
    private static ApiHitAndHandle apiHitAndHandle;
    private ProgressDialog progressDialog;
    private ApiResponse apiResponse;

    public static ApiHitAndHandle getInstance() {
        if (apiHitAndHandle == null) {
            apiHitAndHandle = new ApiHitAndHandle();
        }
        return apiHitAndHandle;
    }

    public void updateProgressDialog(ProgressDialog progressDialog) {
        this.progressDialog = progressDialog;
    }

    public void makeApiCall(Call call, ApiResponse apiResponse) {
        makeApiCall(call, true, apiResponse);
    }

    public void makeApiCall(Call call, boolean showProgress, ApiResponse apiResponse) {
        try {
            this.apiResponse = apiResponse;
            call.enqueue(this);
            if (showProgress) {
                progressDialog.startProgressDialog();
            }
            if (BuildConfig.DEBUG) {
                log("Url to hit " + call.request().url() + "");
                log("Post Params \n" + bodyToString(call.request().body())
                        .replace("\r", "")
                        .replaceAll("--+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                        .replaceAll("Content+[a-zA-Z0-9-\\/:=;\\ ]+;\\s", "")
                        .replaceAll("Content+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                        .replaceAll("charset+[a-zA-Z0-9-\\/:=;\\ ]+\\n", "")
                        .replace("name=", "")
                        .replace("\n\n", "--> ")
                );
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void log(String s) {
        if (BuildConfig.DEBUG)
            Log.e(TAG, ">>>> " + s);
    }

    private String bodyToString(RequestBody request) {
        try {
            Buffer buffer = new Buffer();
            if (request != null)
                request.writeTo(buffer);
            else
                return "";
            return buffer.readUtf8();
        } catch (IOException e) {
            return "did not work";
        }
    }

    @Override
    public void onResponse(Call call, Response response) {
        progressDialog.stopProgressDialog();
        try {
            if (response.isSuccessful()) {
                Object o = response.body();
                apiResponse.onSuccess(call, o);
            } else {
                JsonParser parser = new JsonParser();
                JsonElement mJson = parser.parse(response.errorBody().string());
                Gson gson = new Gson();
                ErrorData errorResponse = gson.fromJson(mJson, ErrorData.class);
                log(errorResponse.message + "");
                apiResponse.onError(call, response.code(), errorResponse.message, apiResponse);
            }
        } catch (Exception ignored) {
        }
    }

    @Override
    public void onFailure(Call call, Throwable t) {
        progressDialog.stopProgressDialog();
        try {
            if (t.getClass() == SocketTimeoutException.class) {
                apiResponse.onError(call, 500, "socketTimeout", apiResponse);
            } else {
                apiResponse.onError(call, 500, t.getMessage(), apiResponse);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
