package com.vendor.humfuel.fragment

import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v7.app.AlertDialog
import android.view.View
import android.widget.AdapterView
import android.widget.CompoundButton
import android.widget.TextView
import com.vendor.humfuel.activity.BaseActivity
import com.vendor.humfuel.activity.MainActivity
import com.vendor.humfuel.apiHandle.ApiClient
import com.vendor.humfuel.apiHandle.ApiHitAndHandle
import com.vendor.humfuel.apiHandle.ApiInterface
import com.vendor.humfuel.apiHandle.ApiResponse
import retrofit2.Call


/**
 * Created by Neeraj Narwal\neeraj.narwal on 2/2/16.
 */
open class BaseFragment : Fragment(), AdapterView.OnItemClickListener, View.OnClickListener, AdapterView.OnItemSelectedListener, CompoundButton.OnCheckedChangeListener, ApiResponse {
    lateinit var apiInterface: ApiInterface
    lateinit var apiHitAndHandle: ApiHitAndHandle
    lateinit var baseActivity: BaseActivity

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        baseActivity = activity as BaseActivity
        baseActivity.hideSoftKeyboard()
        apiHitAndHandle = ApiHitAndHandle.getInstance()
        apiHitAndHandle.updateProgressDialog(baseActivity.progressDialog)
        apiInterface = ApiClient.getClient().create(ApiInterface::class.java)
    }

    fun setToolbar(title: String, show: Boolean) {
        (baseActivity as MainActivity).setToolbar(title, show)
    }

    protected fun getText(textView: TextView): String {
        return textView.text.toString().trim { it <= ' ' }
    }

    override fun onResume() {
        super.onResume()
        activity?.invalidateOptionsMenu()
    }

    override fun onClick(v: View) {

    }

    fun showToast(msg: String) {
        baseActivity.showToast(msg, false)
    }

    fun showToast(msg: String?, isError: Boolean) {
        baseActivity.showToast(msg, isError)
    }

    fun log(s: String) {
        baseActivity.log(s)
    }

    fun gotoMainAct() {
        startActivity(Intent(baseActivity, MainActivity::class.java))
        baseActivity.finish()
    }

    override fun onItemClick(parent: AdapterView<*>, view: View, position: Int, id: Long) {

    }

    override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {

    }

    override fun onNothingSelected(parent: AdapterView<*>) {

    }

    override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {

    }

    fun isValidMail(email: String): Boolean {
        return email.matches("[a-zA-Z0-9._-]+@[a-z]+\\.+[a-z]+".toRegex())
    }

    fun isValidPassword(password: String): Boolean {
        return password.matches("^(?=\\S+$).{8,}$".toRegex())
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, apiResponse: ApiResponse) {
        if (errorMessage != null && errorMessage == "socketTimeout") {
            showRetry(call, apiResponse)
        } else {
            showToast(errorMessage, true)
            call.cancel()
        }
    }

    private fun showRetry(call: Call<*>, apiResponse: ApiResponse) {
        val builder = AlertDialog.Builder(baseActivity)
        builder.setTitle("Alert !1")
        builder.setMessage("Request timeout")
        builder.setPositiveButton("Retry") { dialogInterface, i ->
            val call1 = call.clone()
            apiHitAndHandle.makeApiCall(call1, apiResponse)
        }
        builder.show()
    }
}
