package com.vendor.humfuel.fragment

import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.vendor.humfuel.R
import com.vendor.humfuel.apiHandle.ApiResponse
import com.vendor.humfuel.data.OrderData
import com.vendor.humfuel.utils.Const
import kotlinx.android.synthetic.main.inc_selector_button.*
import retrofit2.Call
import java.util.*

/**
 * Created by neeraj on 14/6/17.
 */
class HomeFragment : BaseFragment() {

    private var orderCall: Call<ArrayList<OrderData>>? = null
    private val completedOrders = ArrayList<OrderData>()
    private val pendingOrders = ArrayList<OrderData>()
    private var orderIdToShowOnTop: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (arguments != null && arguments!!.containsKey("orderId")) {
            orderIdToShowOnTop = arguments!!.getString("orderId")
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        setToolbar(getString(R.string.app_name), true)
        return inflater.inflate(R.layout.fg_home, null)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
        getOrdersList()
    }

    private fun initUI() {
        text1TV.setOnClickListener { setLayout(1) }
        text2TV.setOnClickListener { setLayout(2) }
    }

    fun getOrdersList() {
        orderCall = apiInterface.getOrderList(baseActivity.store.getInt(Const.USER_ID).toString())
        apiHitAndHandle.makeApiCall(orderCall, this)
    }

    private fun setLayout(i: Int) {
        text1TV.setBackgroundColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.White else R.color.transparent))
        text2TV.setBackgroundColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.transparent else R.color.White))
        text1TV.setTextColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.Black else R.color.White))
        text2TV.setTextColor(ContextCompat.getColor(baseActivity, if (i == 1) R.color.White else R.color.Black))
        val bundle = Bundle()
        bundle.putParcelableArrayList("dataList", if (i == 1) pendingOrders else completedOrders)
        val fragment = if (i == 1) PendingOrderFragment() else CompletedOrderFragment()
        fragment.setArguments(bundle)
        baseActivity.supportFragmentManager
                .beginTransaction()
                .replace(R.id.frame_container_tabs, fragment)
                .commit()
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        super.onSuccess(call, `object`)
        pendingOrders.clear()
        completedOrders.clear()
        val orderDatas = `object` as ArrayList<OrderData>
        for (orderData in orderDatas) {
            if (orderData.orderStatus == Const.OrderStatus.ASSIGNED || orderData.orderStatus == Const.OrderStatus.DISPATCHED) {
                pendingOrders.add(orderData)
            } else {
                completedOrders.add(orderData)
            }
        }
        if (orderIdToShowOnTop != null) {
            var foundInCompletes = false
            for (orderData in completedOrders) {
                if (orderIdToShowOnTop == orderData.id.toString()) {
                    completedOrders.remove(orderData)
                    completedOrders.add(0, orderData)
                    foundInCompletes = true
                    break
                }
            }
            if (!foundInCompletes) {
                for (orderData in pendingOrders) {
                    if (orderIdToShowOnTop == orderData.id.toString()) {
                        pendingOrders.remove(orderData)
                        pendingOrders.add(0, orderData)
                        break
                    }
                }
                text1TV!!.callOnClick()
            } else
                text2TV!!.callOnClick()
        } else
            text1TV!!.callOnClick()
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, apiResponse: ApiResponse) {
        if (errorMessage != "No orders found!") {
            super.onError(call, statusCode, errorMessage, apiResponse)
        }
        text1TV!!.callOnClick()
    }
}
