package com.vendor.humfuel.fragment

import android.graphics.Bitmap
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.vendor.humfuel.R
import com.vendor.humfuel.adapter.OrdersAdapter
import com.vendor.humfuel.apiHandle.ApiResponse
import com.vendor.humfuel.data.OrderData
import com.vendor.humfuel.utils.ImageUtils
import kotlinx.android.synthetic.main.fg_home_ordelist.*
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Call
import java.util.*

/**
 * Created by neeraj on 4/8/17.
 */
class PendingOrderFragment : BaseFragment() {

    private var orderDatas: ArrayList<OrderData>? = ArrayList()
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (arguments != null && arguments!!.containsKey("dataList")) {
            orderDatas = arguments!!.getParcelableArrayList("dataList")
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fg_home_ordelist, null)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initUI()
    }

    private fun initUI() {
        listRV.setLayoutManager(LinearLayoutManager(baseActivity))
        emptyTV.setVisibility(if (orderDatas!!.size == 0) View.VISIBLE else View.GONE)
        if (orderDatas!!.size > 0) {
            val ordersAdapter = OrdersAdapter(baseActivity, orderDatas!!, this)
            listRV.setAdapter(ordersAdapter)
        }
    }

    fun dispatchOrder(orderData: OrderData) {
        apiHitAndHandle.makeApiCall(apiInterface.orderDispatch(orderData.id.toString()), this)
    }

    fun completeOrder(bitmap: Bitmap, orderData: OrderData) {
        val file = ImageUtils.bitmapToFile(bitmap, baseActivity)
        log("File size >>>>>>>>>>> " + file.length())
        val filePart = MultipartBody.Part.createFormData("FileName", file.name, RequestBody.create(MediaType.parse("image/*"), file))
        apiHitAndHandle.makeApiCall(apiInterface.orderComplete(orderData.id.toString(), filePart), this)
    }

    fun cancelOrder(cancelReason: String, orderData: OrderData) {
        apiHitAndHandle.makeApiCall(apiInterface.orderCancel(orderData.id.toString(), cancelReason), this)
    }

    override fun onSuccess(call: Call<*>, `object`: Any) {
        super.onSuccess(call, `object`)
        showToast("Status Updated")
        val fragment = baseActivity.supportFragmentManager.findFragmentById(R.id.frame_container)
        if (fragment is HomeFragment) {
            fragment.getOrdersList()
        }
    }

    override fun onError(call: Call<*>, statusCode: Int, errorMessage: String?, apiResponse: ApiResponse) {
        super.onError(call, statusCode, errorMessage, apiResponse)
        val fragment = baseActivity.supportFragmentManager.findFragmentById(R.id.frame_container)
        if (fragment is HomeFragment) {
            fragment.getOrdersList()
        }
    }
}
